/********************************
 * config.c
 * コンフィグおよび各モジュールの設定
 *
 * ******************************
 */

#include <xc.h>
#include "config.h"

// Configuration bits *****************
#pragma config FEXTOSC = OFF,RSTOSC = HFINT1  // HFINTOSC (1MHz)
#pragma config CLKOUTEN = OFF,CSWEN = ON,FCMEN = OFF
#pragma config MCLRE = OFF,PWRTE = OFF,WDTE = OFF,LPBOREN = OFF
#pragma config BOREN = OFF,BORV = LOW,PPS1WAY = OFF,STVREN = ON
#pragma config DEBUG = OFF
#pragma config WRT = OFF,LVP = OFF,CP = OFF,CPD = OFF

void int_pins(){
    ANSELA = 0;                 // 全てデジタルIO
    ANSELC = 0;                 // 全てデジタルIO
    WPUA = 0b111111;            // 入力は全て弱プルアップ
    WPUC = 0b111111;            // 入力は全て弱プルアップ
    TRISA = 0b101111;           // RA4 Sp出力
    TRISC = 0b000011;           // RC0 LED出力
    LATA = 0;
    LATC = 0;                   // LED 消灯

    // Pin 状態変化 I-O-C設定 -----------------------------------
    IOCAN5 = 1;             // 立下がり検出
    IOCAP5 = 1;             // 立上がり検出
    IOCAF5 = 0;             // 入力変位フラッグ クリア
    IOCIE  = 1;             // 割り込み許可

}

void int_modules(){
    // Timer 0 設定 -------------------------------------------
    // Fosc/4 PS/4 /2^16 = 131mS(countUp) Post 1/4 = 524mS
    // LED点滅の時間設定に使用する
    T0CON1 = 0b01000001;    // Fosc/4 Sync PreS 1/2
    T0CON0 = 0b00010100;    // T0 ON 16bit Post 1/4


    // Timer 1 設定 -------------------------------------------
    // リモコンパルス幅の計測に使用する Timer は、1カウント 1uS
    T1CON  = 0b00010000;    // Timer 1 FOSC/4 1/2(66mSecごと)
    T1GCON = 0b00000000;    // Timer 1 Gate設定(使用せず)

    // CCP Timer 連携設定 ***********************************
    CCPTMRS = 0b11101001;       // CCP4-CCP3-CCP2-CCP1
                                // TMR5-TMR3-TMR3-TMR2
    // CCP1 PWM Timer2 設定 (音量制御) ***********************
    //   8MHz/4/64 = 31KHz
    // *****************************************************
    T2CON  = 0b00000100;        // Timer 2 PS1/1設定
    PR2    = 0x3F;              // Timer2 Period Register設定
    CCPR1H = 0;                 // PR2が0x3F duty右詰のため
    CCPR1L = 0;                 // CCPR1Lの8bitだけが有効
    CCP1CON = 0b10001111;       // PWN duty右詰め

    // Timer3 の設定 (音符OnOff制御) **************************
    // LFintOsc 31KHz PS:/8 = 3.9KHz (0.258mS毎)
    // *****************************************************
    T3CON = 0b11110001;          // LFintOsc PreS 1/8
    T3GCON = 0;

    // CCP2 Timer3連携 Compare設定 (無音時間) *****************
    //   無音時間 30mS = 116 (0x74)  -50mS = -194 (0xFF3E)
    // *****************************************************
    CCPR2H = 0;                 //
    CCPR2L = 116;               //
    CCP2CON = 0b10001010;       // Compare
    CCP2IF = 0;

    // CCP3 Timer3連携 Compare設定 (演奏時間) *****************
    //   音符継続時間 500mS = 1938 (0x0792)
    // *****************************************************
    CCPR3H = 0x07;              //
    CCPR3L = 0x92;              //
    CCP3CON = 0b10001010;       // Compare
    CCP3IF = 0;

    // CCP4 Timer5連携 Compare設定 (音量減衰時間) **************
    //   Timer5 : LFintOsc 31KHz PS:/8 = 3.9KHz (0.258mS)
    //   音符音量減衰時間 50mS = 194
    // *****************************************************
    T5CON = 0b11110001;         // LFintOsc PreS 1/8
    T5GCON = 0;
    CCPR4H = 0;                 //
    CCPR4L = 194;               //
    CCP4CON = 0b10001011;       // Compare with reset TMR5
    CCP4IF = 0;
    

    // Timer4 の設定 1mS毎の時間設定 **************************
    // 8MHz /4 PS:/64 PR4;/250 POSt/10 = 80mS毎
    // *****************************************************
//    T4CON = 0b01001011;      // Timer OFF PS 1/64 POST 1/10
//    PR4      = 249;          // 繰返し周波数 125Hz
//    TMR4IF = 0;              // Timer4フラグクリア
//    TMR4IE = 1;              // Timer4フラグクリア

    // Timer6 の設定 NCO用のクロック **************************
    // 8MHz /4 PS:/1 PR6;/8 = 250KHz
    // *****************************************************
    T6CON = 0b00000100;         // Timer ON PreS 1/1
    PR6   = 7;                  // 繰返し周波数 250KHz

    // CLC1 初期化 **************************************
    // AND-ORを使用 入力は、T6_match、そのまま出力、
    // NCOで利用する
    // *************************************************
    CLC1POL  = 0x02;    // Gate2出力反転 他はそのまま
    CLC1SEL0 = 0x23;    // 信号源1 35 = TMR6/PR6一致
    CLC1SEL1 = 0x00;    // 信号源2 CLCIN0PPS　利用なし
    CLC1SEL2 = 0x00;    // 信号源3 CLCIN0PPS　利用なし
    CLC1SEL3 = 0x00;    // 信号源4 CLCIN0PPS　利用なし
    CLC1GLS0 = 0x02;    // Gate1 Data1T ON 他はOFF
    CLC1GLS1 = 0x00;    // Gate2 全入力OFF
    CLC1GLS2 = 0x00;    // Gate3 全入力OFF
    CLC1GLS3 = 0x00;    // Gate4 全入力OFF
    CLC1CON  = 0x80;    // LC1 ON 割込OFF MODE AND-OR
    // CLC3 初期化 **************************************
    // SR latchを使用 S入力は、CCP2、R入力は、CCP3
    // CLC4で利用する
    // *************************************************
    CLC3POL = 0x00;         // 反転は無し
    CLC3SEL0 = 0x0D;        // 信号源1 CCP2 output;
    CLC3SEL1 = 0x0E;        // 信号源2 CCP3 output;
    CLC3SEL2 = 0x00;        // 信号源3
    CLC3SEL3 = 0x00;        // 信号源4
    CLC3GLS0 = 0x02;        // Gate1 Data1T ON 他はOFF
    CLC3GLS1 = 0x00;        // Gate2
    CLC3GLS2 = 0x08;        // Gate3 Data2T ON 他はOFF
    CLC3GLS3 = 0x00;        // Gate4
    CLC3CON = 0x83;         // LC3EN SR latch;
//    RC0PPS = 6;             // RC0にCLC3出力

    // CLC4 初期化 **************************************
    // AND-ORを使用 入力は、DSM と CLC3
    // 出力はオルゴール出力
    // *************************************************
    CLC4POL = 0x00;         // 反転は無し
    CLC4SEL0 = 0x0A;        // LC4D1S DSM_OUT;
    CLC4SEL1 = 0x06;        // LC4D2S LC3_out;
    CLC4SEL2 = 0x00;        //
    CLC4SEL3 = 0x00;        //
    CLC4GLS0 = 0x02;        // LC4G1D1T enabled
    CLC4GLS1 = 0x08;        // LC4G2D2T enabled
    CLC4GLS2 = 0x00;        //
    CLC4GLS3 = 0x00;        //
    CLC4CON = 0x80;         // LC4EN  AND-OR;
    RA4PPS  = 7;            // RA4にCLC4出力

    // NCO 設定 ************************************************
    NCO1CLK = 0b00000010;       // CLC1-outがクロック
    NCO1CON = 0b10000000;       // 正論理、反転モード
    NCO1INC = 0;

    // DSM 設定 *********************************************
    //   PWM5搬送波信号とNCO情報信号を混合
    // *****************************************************
    MDCON = 0x80;               // DSMは『On/Off』モード
    MDSRC = 0x0B;               // 情報信号 正論理非同期 NOC1
    MDCARH = 0x04;              // 搬送波Ｈ 正論理非同期 CCP1
    MDCARL = 0;                 // 搬送波Ｌ VSS
}
