#include <xc.h>
#include <stdint.h>
#include <stdlib.h>
#include "config.h"
#include "song.h"
#include "led.h"

void modeLED(void){     // LEDモードに設定
    boxMode = mSteady;  // boxModeをLEDに変更
    LATC = 0x3F;        // 変更を表示するためLEDを二度点灯
    __delay_ms(50);     // 50mS 点灯
    LATC = 0;           // 
    __delay_ms(150);    // 150mS 消灯
    LATC = 0x3F;        //
    __delay_ms(50);     // 50mS 点灯
    LATC = 0;           // 消灯
}
// ---------------------------------------------------
void steady(uint8_t ptn){   // 静的点灯Mode処理
    boxMode = mSteady;      // 静的Mode　保存
    T0EN   = 0;             // Timer0 Timer5 停止
    TMR0IF = 0;
    TMR5ON = 0;
    CCP4IF = 0;
    switch(ptn){            // 引数から、点灯・消灯するLED処理
        case 0: LATC = 0; break;   // 全消灯
        case 1: LED1 ^= 1; break;  // LED1 反転
        case 2: LED2 ^= 1; break;  // LED2 反転
        case 3: LED3 ^= 1; break;  // LED3 反転
        case 4: LED4 ^= 1; break;  // LED4 反転
        case 9: LATC ^= 0x3F; break;  // 全反転
    }
}
void flash(uint8_t ptn){    // 動的点灯Mode処理
    boxMode = mFlash;       // 動的Mode 保存
    ptnNum = ptn;           // 点滅パターン設定
    ptnIdx = 1;             // パターン位置設定
    TMR5ON = 0;             // Timer5 停止
    CCP4IF = 0;
    TMR0H = 0xFF;           // Timer0 起動
    TMR0L = 0xF0;
    T0EN  = 1;
    TMR0IF = 0;
    TMR0IE = 1;
}

void random(void){          // 乱数点灯Mode処理
    boxMode = mRandum;
    T0EN  = 1;              // Timer0 起動
    TMR0IF = 0;
    TMR0IE = 1;
}
// LED点灯パターンの変化スピードを可変する
// Timer0のPostScalerを1−15の範囲で変化させる
void speed(uint8_t plus){
    uint8_t val = T0CON0 & 0xF;
    if(plus){
        if(val > 0)val--;
    }else{
        if(val < 0xF)val++;
    }
    T0CON0 = (T0CON0 & 0xF0) | val;
}
// 点滅方向の変更 乱数Modeでは、点灯する閾値を変更
void dir(uint8_t way){
    ptnDir = way;
    if(way)onTh = 22000;
    else onTh = 15000;
}

// 割り込み処理 led_isr **********************
void led_isr(void){
    if(boxMode == mRandum){
        //乱数点灯Mode --------------------
        uint8_t randDt=0;
        if(rand() > onTh)randDt  = 1;
        if(rand() > onTh)randDt += 2;
        if(rand() > onTh)randDt += 4;
        if(rand() > onTh)randDt += 8;
        LATC=randDt << 2;
        TMR5=0;         // 50mSだけ点灯するため
        TMR5ON = 1;     // TMR5-CCP4を有効にする
        CCP4IF=0;
        CCP4IE=1;
    }else{
        //規定のパターに従い点滅を制御 --------------------
        if(ptnDir){
            LATC = LEDptn[ptnNum][ptnIdx++] << 2;
            if(ptnIdx > LEDptn[ptnNum][0])ptnIdx = 1;
        }else{
            LATC = LEDptn[ptnNum][ptnIdx--] << 2;
            if(ptnIdx == 0)ptnIdx =  LEDptn[ptnNum][0];
        }
    }
}
