
#include <xc.h>
#include <stdint.h>
#include "song.h"
#include "led.h"
#include "gakufu.h"

void modeSong(void){            // Songモードに設定
    boxMode = mSong;            // モード表示セット
    LATC = 0;                   // LED消灯
    T0EN   = 0;                 // Timer0 OFF
    TMR0IF = 0;                 // 
    TMR5ON = 1;                 // FlashTimer ON
    CCP4IE = 0;                 // 
    CCP4IF = 0;                 // 
    //
    NCO1INCH = 0x0D;            // NCO1INCに「ラ」セット
    NCO1INCL = 0x9B;
    RA4PPS  = 29;               // RA4にNCO出力
    __delay_ms(100);            // 100mS
    TRISA4 = 1;                 // 音 OFF
    __delay_ms(100);            // 100mS
    TRISA4 = 0;                 // 音 ON
    __delay_ms(100);            // 100mS
    RA4PPS  = 7;                // RA4はCLC4出力
}
// **********************************************
// 指定番号の楽譜を演奏する 
// **********************************************
void playSong(uint8_t num){     
    uint16_t cntOnpu;           // 音符再生時間
    uint16_t incWave;           // 音符周波数
    uint16_t pOnp = 0;          // 演奏音符位置
    uint16_t Onp;               // 音符
    uint8_t state = 0;          // 制御ステート
    uint8_t tempo;              // テンポ
    int16_t maxVR;              // 最大音量
    // 指定楽譜の先頭にポインタ(pOnp)をセットする
    num--;          
    if(num){
        while(num--){
            while(p1gkf[pOnp++]!=0xFFFF);
            if(p1gkf[pOnp]==0xFFFF){
                pOnp = 0;
//                num = 0;
                break;
            }
        }
    }
    // 繰り返し -----------------------------------------
    while(1){
        if(revdData){                   // 赤外線コマンドを受信した
            switch(revdData){               // コマンドで分類
                case 0xC078:                // key 大
                    maxVR += 30;            //   音量を＋30
                    if(maxVR>255)maxVR=255; //   最大値 255
                    break;
                case 0xC0F8:                // key 小
                    maxVR -= 30;            //   音量を-30
                    if(maxVR<30)maxVR=30;   //   最小値 30
                    break;
                case 0xC098:                // key ＝
                    tempo -= 10;            //  テンポを-10
                    if(tempo<30)tempo=30;   //  最小値 30
                    break;
                case 0xC018:                // key ー
                    tempo += 10;            //  テンポを+10
                    if(tempo>150)tempo=150; //  最大値 150
                    break;
                default:                    // 上記以外のKey
                    state = gkfEnd;         //  終了処理へ
                    break;
            }
            revdData = 0;
        }
        //------------------- 主フロー制御 ---------------
        switch(state){
            case waitStart:             // ====== 演奏開始待ち ====
                state = getGakufu;
                break;
            case getGakufu:             // ====== 楽譜の設定 ======
                if(p1gkf[pOnp]==0xFFFF)pOnp=0;  // 楽譜終了なら先頭に
                tempo = 7500/p1gkf[pOnp++];     // 楽譜のテンポ取得
                playCnt = 0;                    // 演奏時間リセット
                maxVR = 0xFF;                   // 音量最大
                state = getOnpu;
                break;
            case getOnpu:               // ====== 音符の設定 ======
                Onp = p1gkf[pOnp++];            // 音符を取得
                if(Onp == 0xFFFF){              // 曲の終了なら
                    state = gkfEnd;             // 抜ける
                    break;
                }
                cntOnpu = Onp & 0x00FF;         // 音符の長さ取得
                cntOnpu *= tempo * 4;           // 音符の長さ取得
                Onp >>= 8;                      // 音高を取り出す
                if(Onp){                        // 音符なら
                    incWave = dW_tbl[Onp - 23]; //  音高を繰返時間に変換
                    musicOn();                  //  出力をON
                    volume = maxVR;             //  音量最大
                }else{                          // 休符なら
                    incWave = 0;                //  繰返し　なし
                    musicOff();                 //  出力OFF
                    volume = 0;                 //  音量OFF
                }
                NCO1INCH = incWave >> 8;        // NCO1INCに結果セット
                NCO1INCL = incWave;
                playLen = cntOnpu;              // 演奏時間をセット
                playEnd = 0;                    // 演奏終了フラグをリセット
                state = onpuEnd;
                break;
            case onpuEnd:             // ====== 音符の演奏終了 ======
                if(reduceFlg){                  // 50mS毎に
                    reduceFlg = 0;              // 音量を -6% する
                    volume = (int)volume * 15/16;
                }
                if(playEnd){                    // 音符演奏終了を待つ
                    playCnt = 0;                // 演奏経過時間をリセット
                    state = getOnpu;            // 次の音符取得へ
                }
                break;
            case gkfEnd:              // ====== 楽譜の演奏終了 ======
                musicOff();                     // 音量OFF
                return;
                break;
        }
    }   //  while

}
